@@ -1,11 +1,12 @@
 /* gpgv.c - The GnuPG signature verify utility
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2005, 2006,
+ *               2009, 2012 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -28,6 +28,10 @@
 #ifdef HAVE_DOSISH_SYSTEM
 #include <fcntl.h> /* for setmode() */
 #endif
+#ifdef HAVE_LIBREADLINE
+#include <stdio.h>
+#include <readline/readline.h>
+#endif
 
 #define INCLUDED_BY_MAIN_MODULE 1
 #include "packet.h"
@@ -44,7 +48,7 @@
 #include "ttyio.h"
 #include "i18n.h"
 #include "status.h"
-#include "g10defs.h"
+#include "cardglue.h"
 
 
 enum cmd_and_opt_values { aNull = 0,
@@ -52,7 +56,7 @@ enum cmd_and_opt_values { aNull = 0,
     oVerbose	  = 'v',
     oBatch	  = 500,
     oKeyring,
-    oIgnoreTimeConflict,                      
+    oIgnoreTimeConflict,
     oStatusFD,
     oLoggerFD,
     oHomedir,
@@ -66,7 +70,7 @@ static ARGPARSE_OPTS opts[] = {
     { oVerbose, "verbose",   0, N_("verbose") },
     { oQuiet,	"quiet",     0, N_("be somewhat more quiet") },
     { oKeyring, "keyring"   ,2, N_("take the keys from this keyring")},
-    { oIgnoreTimeConflict, "ignore-time-conflict", 0, 
+    { oIgnoreTimeConflict, "ignore-time-conflict", 0,
                            N_("make timestamp conflicts only a warning") },
     { oStatusFD, "status-fd" ,1, N_("|FD|write status info to this FD") },
     { oLoggerFD, "logger-fd",1, "@" },
@@ -78,10 +82,6 @@ static ARGPARSE_OPTS opts[] = {
 
 int g10_errors_seen = 0;
 
-#ifdef __riscos__
-RISCOS_GLOBAL_STATICS("GnuPG (gpgv) Heap")
-#endif /* __riscos__ */
-
 const char *
 strusage( int level )
 {
@@ -99,7 +99,7 @@ strusage( int level )
 	    _("Usage: gpgv [options] [files] (-h for help)");
 	break;
       case 41:	p =
-	    _("Syntax: gpg [options] [files]\n"
+	    _("Syntax: gpgv [options] [files]\n"
 	      "Check signatures against known trusted keys\n");
 	break;
 
@@ -109,33 +109,17 @@ strusage( int level )
 }
 
 
-/* We need the home directory also in some other directories, so make
-   sure that both variables are always in sync. */
-static void
-set_homedir (char *dir)
-{
-  if (!dir)
-    dir = "";
-  g10_opt_homedir = opt.homedir = dir;
-}
-
-
 
 
 static void
 i18n_init(void)
 {
 #ifdef USE_SIMPLE_GETTEXT
-    set_gettext_file( PACKAGE );
+    set_gettext_file (PACKAGE, "Software\\GNU\\GnuPG");
 #else
 #ifdef ENABLE_NLS
-#ifdef HAVE_LC_MESSAGES
-       setlocale( LC_TIME, "" );
-       setlocale( LC_MESSAGES, "" );
-#else
-       setlocale( LC_ALL, "" );
-#endif
-    bindtextdomain( PACKAGE, G10_LOCALEDIR );
+    setlocale( LC_ALL, "" );
+    bindtextdomain (PACKAGE, LOCALEDIR);
     textdomain( PACKAGE );
 #endif
 #endif
@@ -151,33 +135,23 @@ main( int argc, char **argv )
     STRLIST nrings=NULL;
     unsigned configlineno;
 
-#ifdef __riscos__
-    riscos_global_defaults();
-#endif /* __riscos__ */
-
     log_set_name("gpgv");
     init_signals();
     i18n_init();
     opt.command_fd = -1; /* no command fd */
     opt.pgp2_workarounds = 1;
-    opt.keyserver_options.auto_key_retrieve = 1;
-    opt.always_trust = 1;
+    opt.keyserver_options.options|=KEYSERVER_AUTO_KEY_RETRIEVE;
+    opt.trust_model = TM_ALWAYS;
     opt.batch = 1;
 
-#if defined (_WIN32)
-    set_homedir (read_w32_registry_string( NULL, "Software\\GNU\\GnuPG", "HomeDir" ));
-#else
-    set_homedir (getenv("GNUPGHOME"));
-#endif
-    if( !*opt.homedir ) {
-      set_homedir (GNUPG_HOMEDIR);
-    }
+    opt.homedir = default_homedir ();
+
     tty_no_terminal(1);
     tty_batchmode(1);
-    disable_dotlock();
+    dotlock_disable ();
 
     set_native_charset (NULL); /* Try to auto set the character set */
-    
+
     pargs.argc = &argc;
     pargs.argv = &argv;
     pargs.flags=  1;  /* do not remove the args */
@@ -189,7 +163,7 @@ main( int argc, char **argv )
           case oKeyring: append_to_strlist( &nrings, pargs.r.ret_str); break;
 	  case oStatusFD: set_status_fd( pargs.r.ret_int ); break;
 	  case oLoggerFD: log_set_logfile( NULL, pargs.r.ret_int ); break;
-	  case oHomedir: set_homedir (pargs.r.ret_str); break;
+	  case oHomedir: opt.homedir = pargs.r.ret_str; break;
 	  case oIgnoreTimeConflict: opt.ignore_time_conflict = 1; break;
 	  default : pargs.err = 2; break;
 	}
@@ -198,16 +172,18 @@ main( int argc, char **argv )
     if( log_get_errorcount(0) )
 	g10_exit(2);
 
+    g10_opt_homedir = opt.homedir;
+
     if( opt.verbose > 1 )
 	set_packet_list_mode(1);
 
     if( !nrings )  /* no keyring given: use default one */
-        keydb_add_resource ("trustedkeys" EXTSEP_S "gpg", 0, 0);
+        keydb_add_resource ("trustedkeys" EXTSEP_S "gpg", 8, 0);
     for(sl = nrings; sl; sl = sl->next )
-        keydb_add_resource (sl->d, 0, 0 );
-    
+        keydb_add_resource (sl->d, 8, 0 );
+
     FREE_STRLIST(nrings);
-    
+
     if( (rc = verify_signatures( argc, argv ) ))
         log_error("verify signatures failed: %s\n", g10_errstr(rc) );
 
@@ -227,7 +203,7 @@ g10_exit( int rc )
 
 
 /* Stub:
- * We have to override the trustcheck from pkclist.c becuase 
+ * We have to override the trustcheck from pkclist.c becuase
  * this utility assumes that all keys in the keyring are trustworthy
  */
 int
@@ -237,9 +213,11 @@ check_signatures_trust( PKT_signature *sig )
 }
 
 void
-read_trust_options(byte *trust_model,ulong *created,ulong *nextcheck) {}
+read_trust_options(byte *trust_model,ulong *created,ulong *nextcheck,
+		   byte *marginals,byte *completes,byte *cert_depth,
+		   byte *min_cert_level) {}
 
-/* Stub: 
+/* Stub:
  * We don't have the trustdb , so we have to provide some stub functions
  * instead
  */
@@ -250,13 +228,33 @@ cache_disabled_value(PKT_public_key *pk)
   return 0;
 }
 
+void
+check_trustdb_stale(void) {}
+
 int
 get_validity_info (PKT_public_key *pk, PKT_user_id *uid)
 {
     return '?';
 }
 
-/* Stub: */
+unsigned int
+get_validity (PKT_public_key *pk, PKT_user_id *uid)
+{
+  return 0;
+}
+
+const char *
+trust_value_to_string (unsigned int value)
+{
+  return "err";
+}
+
+const char *
+uid_trust_string_fixed(PKT_public_key *key,PKT_user_id *uid)
+{
+  return "err";
+}
+
 int
 get_ownertrust_info (PKT_public_key *pk)
 {
@@ -270,16 +268,35 @@ get_ownertrust (PKT_public_key *pk)
 }
 
 
-/* Stub:
+/* Stubs:
  * Because we only work with trusted keys, it does not make sense to
  * get them from a keyserver
  */
+
+struct keyserver_spec *
+keyserver_match(struct keyserver_spec *spec) { return NULL; }
+
 int
 keyserver_import_keyid( u32 *keyid, void *dummy )
 {
     return -1;
 }
 
+int
+keyserver_import_cert(const char *name) { return -1; }
+
+int
+keyserver_import_pka(const char *name,unsigned char *fpr) { return -1; }
+
+int
+keyserver_import_name(const char *name,struct keyserver_spec *spec)
+{
+  return -1;
+}
+
+int
+keyserver_import_ldap(const char *name) { return -1; }
+
 /* Stub:
  * No encryption here but mainproc links to these functions.
  */
@@ -320,7 +337,7 @@ check_secret_key( PKT_secret_key *sk, int n )
 }
 
 /* Stub:
- * No secret key, so no passphrase needed 
+ * No secret key, so no passphrase needed
  */
 DEK *
 passphrase_to_dek( u32 *keyid, int pubkey_algo,
@@ -332,11 +349,25 @@ passphrase_to_dek( u32 *keyid, int pubkey_algo,
   return NULL;
 }
 
+struct keyserver_spec *parse_preferred_keyserver(PKT_signature *sig) {return NULL;}
+struct keyserver_spec *parse_keyserver_uri(const char *uri,int require_scheme,
+                                           const char *configname,
+                                           unsigned int configlineno)
+{
+  return NULL;
+}
+
+void free_keyserver_spec(struct keyserver_spec *keyserver) {}
+
 /* Stubs to avoid linking to photoid.c */
 void show_photos(const struct user_attribute *attrs,int count,PKT_public_key *pk) {}
 int parse_image_header(const struct user_attribute *attr,byte *type,u32 *len) {return 0;}
 char *image_type_to_string(byte type,int string) {return NULL;}
 
+#ifdef ENABLE_CARD_SUPPORT
+int agent_scd_getattr (const char *name, struct agent_card_info_s *info) {return 0;}
+#endif /* ENABLE_CARD_SUPPORT */
+
 /* Stubs to void linking to ../cipher/cipher.c */
 int string_to_cipher_algo( const char *string ) { return 0; }
 const char *cipher_algo_to_string( int algo ) { return "?";}
@@ -358,6 +389,7 @@ void cipher_sync( CIPHER_HANDLE c ) {}
 void random_dump_stats(void) {}
 int quick_random_gen( int onoff ) { return -1;}
 void randomize_buffer( byte *buffer, size_t length, int level ) {}
+void randomize_mpi (MPI mpi, size_t nbits, int level) {}
 int random_is_faked() { return -1;}
 byte *get_random_bits( size_t nbits, int level, int secure ) { return NULL;}
 void set_random_seed_file( const char *name ) {}
@@ -378,18 +410,32 @@ void rndlinux_constructor(void) {}
 /* Stubs to avoid linking to ../util/ttyio.c */
 int tty_batchmode( int onoff ) { return 0; }
 void tty_printf( const char *fmt, ... ) { }
-void tty_print_string( byte *p, size_t n ) { }
-void tty_print_utf8_string( byte *p, size_t n ) {}
-void tty_print_utf8_string2( byte *p, size_t n, size_t max_n ) {}
+void tty_fprintf (FILE *fp, const char *fmt, ... ) { }
+void tty_print_string( const byte *p, size_t n ) { }
+void tty_print_utf8_string( const byte *p, size_t n ) {}
+void tty_print_utf8_string2( const byte *p, size_t n, size_t max_n ) {}
 char *tty_get( const char *prompt ) { return NULL;}
 char *tty_get_hidden( const char *prompt ) {return NULL; }
 void tty_kill_prompt(void) {}
 int tty_get_answer_is_yes( const char *prompt ) {return 0;}
 int tty_no_terminal(int onoff) {return 0;}
+void tty_cleanup_after_signal (void) {}
+#ifdef __VMS
+FILE *ttyfp_is (void) { return stderr; }
+void init_ttyfp (void) { }
+#endif /*__VMS*/
+#ifdef HAVE_LIBREADLINE
+void tty_enable_completion(rl_completion_func_t *completer) {}
+void tty_disable_completion(void) {}
+void rl_cleanup_after_signal (void) {}
+void rl_free_line_state (void) {}
+#endif
 
 /* We do not do any locking, so use these stubs here */
-void disable_dotlock(void) {}
-DOTLOCK create_dotlock( const char *file_to_lock ) { return NULL; }
-int make_dotlock( DOTLOCK h, long timeout ) { return 0;}
-int release_dotlock( DOTLOCK h ) {return 0;}
-void remove_lockfiles(void) {}
+void dotlock_disable(void) {}
+dotlock_t dotlock_create (const char *file_to_lock, unsigned int flags)
+{ return NULL; }
+void dotlock_destroy (dotlock_t h, int reclaim) {}
+int dotlock_take (dotlock_t h, long timeout) { return 0;}
+int dotlock_release (dotlock_t h) {return 0;}
+void dotlock_remove_lockfiles (int reclaim) {}
